function S=isopol3(freq,ang_in,zz0,options)
%
%   isopol2(f,ang,zz,options)
%
%   Gerbrant van Vledder
%   initial version: Feb. 2000 (Oslo Airport)
%
%   Update history
%   xx/xx/xxxx Transition to isopol2
%   29/07/2013 absolute value for delta chosen
%   13/10/2015 Upgrade of routine using options struct
%   30/10/2015 Option headings added
%   15/11/2015 Frequency range clipped in case max(fr)>ffmax
%              Range adpated in case ffmax > max(fr)
%              Verticalalignment of title modified to bottom
%              Option ytfac added to control vertical offset of title
%
%   plot contour plot of 2d-wave spectrum
%
%   freq(:)    frequencies
%   ang_in(:)  directions [if needed converted to radians]
%   zz0(:,:)   2-d array; zz0(nfr,ndir)
%   options
%    .nv       Number of contour lines; default nv=6
%    .cfact    Factor between subsequent contour levels; default cfact=2
%    .ttext    Title Text; default ttext='EXAMPLE'
%    .ffmax    Maximum frequency for scaling; default ffmax=max(freq)
%    .headings Switch to plot Nautical headings, every ihead: default []
%    .inorm    Switch to normalize spectral values: default=1
%    .vv       Contour levels
%    .ytfac    Factor to modify the vertical offset of title wrt fmax
%
%   Future modifications
%   Specify contour levels, to enable same contour heights for partitions
%   put frquency labels at different positions
%
comcolor
%
if nargin==3
  options = [];
end
%
nv       = 6;
cfact    = 2;
ttext    = 'EXAMPLE';
headings = [];
inorm    = 1;
frmax    = max(freq);
ffmax    = frmax;
ytfac    = 1.1;
%
if isfield(options,'nv');      nv       = options.nv;end;
if isfield(options,'cfact');   cfact    = options.cfact;end;
if isfield(options,'ttext');   ttext    = options.ttext;end;
if isfield(options,'ffmax');   ffmax    = options.ffmax;end;
if isfield(options,'headings');headings = options.headings;end;
if isfield(options,'inorm');   inorm    = options.inorm;end;
if isfield(options,'ytfac');   ytfac    = options.ytfac;end
%
%  normalize data
%
zz   = zz0;
if inorm==1
  zmax = max(max(abs(zz)));
  zz   = zz/zmax;
end  
%
%  check range of directions, if needed convert to radian
%
ang  = ang_in;
amax = max(ang);
if amax > 2*pi
  ang = ang*pi/180;
end
%
% make vertical matrix
%
if size(ang,2)>size(ang,1)
  ang = ang';
end
%
% transpose zz
%
zz = zz';
%
% close circle, if not a sector grid
%
delta  = ang(2)-ang(1);
angmin = min(ang);
angmax = max(ang);
angdif = pi-abs(pi-abs(angmin-angmax));
%
if abs(angdif-abs(delta)) <0.02
  ang = [ang ; ang(1) + 2*pi*sign(delta)];
  zz = [zz ; zz(1,:)];
end
%
nfr   = length(freq);
na    = length(ang);
fmin  = min(freq);
fmax  = min(ffmax,frmax);
%
[ff,aa] = meshgrid(freq,ang);
%
if frmax > fmax
  ifmax = find(freq>fmax);
  if1   = min(ifmax)-1;
  if if1>1
%     [fmax freq(if1) freq(if1+1)]
    dfreq = freq(if1+1)-freq(if1);
    w1    = (freq(if1+1)-fmax)/dfreq;
    w2    = (fmax-freq(if1))/dfreq;
    ff    = ff(:,1:if1);
    ff(:,if1) = fmax;
    aa    = aa(:,1:if1);
    zzz   = zz(:,1:if1);
    zzz(:,if1) = w1*zz(:,if1)+w2*zz(:,if1+1);
    zz = zzz;
  else
    disp('ISOPOL3: too low value for fmax')
    pause
  end
end    
% whos ff aa zz
% freq'
% ff(1,:)
% ffmax
% fmax
% ffmax = max(max(ff))
%
% check if data values contain positive and/or negative
% values. This determines which colormaps are used.
%
zmax    = max(max(zz));
zmin    = min(min(zz));
izequal = (zmin == zmax);
%
% construct contour heights, based on
%
zzmax = max(max(abs(zz)));
nv    = max(nv,2);
vv    = [0.5 0.9]*zzmax;
vvmin = 0.5*zzmax;
for iv=3:nv
  vvmin = vvmin/cfact;
  vv    = [vvmin vv];
end
%
if isfield(options,'vv');  vv = options.vv;end;
nv = length(vv);
vv = [-fliplr(vv) vv];
size(vv);

%
% transform to polar coordinates
%
xx = ff.*cos(aa);
yy = ff.*sin(aa);
epsy = max(max(abs(ff)))/1000;
% cosa = cos(ang(1));
%
TextAndLineColor ='k';          % Color used for texts, axes etc.
IsoLineColor ='k';              % Color used for isolines
DoIsoPlotLin = 0;               % Yes/no plotting isolines
DoIsoPlotLab = 0;               % Yes/no plotting isoline labels
printer      = 0;               % Yes/no printer output
%
% plot contour lines of positive part
%
MyCmap = [winter(nv);[1 1 1]; flipud(autumn(nv))];
size(MyCmap);
MyVals = vv;                    % Corresponding values (n)
%
zmax = max(max(zz));
zmin = min(min(zz));
%
nv = length(vv);
if izequal ~= 1  && abs(zmax-zmin)>0.001
  [CS,H] = fcontour(xx,yy,zz,vv);
  flines(CS,epsy,ang(1));
%   contourf(xx,yy,zz,vv)
%   colormap(MyCmap)
end ;
%
% define circles to be plotted
%
if (fmax-fmin)<0.8
  df = 0.1;
else
  df = 0.2;
end
%
cirmin  = max(1,round(fmin/df))*df;
cirmax  = floor(fmax/df)*df;
circles = [fmin cirmin:df:cirmax fmax];
%
alfa = 0:360;
hold on
for ic = 1:length(circles)
  xc = cosd(alfa)*circles(ic);
  yc = sind(alfa)*circles(ic);
  plot(xc,yc,'k--')
end  
%
for beta=0:30:330
  xs = [fmin fmax]*cosd(beta);  
  ys = [fmin fmax]*sind(beta);  
  plot(xs,ys,'k--')
end
%
% title(ttext,'fontsize',8);

text(-fmax/1.3,ytfac*fmax,ttext,'fontsize',12,'verticalalignment','bottom')
set(gca,'xlim',[-fmax fmax],'ylim',[-fmax fmax]);
set(gca,'visible','off')
text(fmax,0.1*fmax,[num2str(fmax) ' Hz'],'backgroundcolor','w','edgecolor','k','fontsize',8)
%
if isempty(headings)==0
  for dd=headings
    alfa = 90-dd;  
    text(1.1*fmax*cosd(alfa),1.1*fmax*sind(alfa),num2str(dd),'horizontalalignment','center')  
  end    
end
%
S.vv = vv;